# USAGE:
#   python  mapdump.py  sourcefile.extension  extensionless_outfile
# E.g.:
# python mapdump.py whitechapel_done.gif london
#   --creates london.dmp and london_turf_stubs.dm
# python mapdump.py australia.png oz
#   --creates oz.dmp and oz_turf_stubs.dm

# WARNINGS:
# --REQUIRES PYTHON (FREELY AVAILABLE FROM PYTHON.ORG)
# --REQUIRES THE PYTHON IMAGING LIBRARY (FREELY AVAILABLE FROM PYTHONWARE.COM)
# --KEEP SOURCE FILES TO 26 COLORS OR LESS
# --BYOND'S MAP SIZE LIMIT IS 1000x1000
# --THIS HAS ONLY BEEN TESTED ON WINDOWS 2000 AND ONLY FOR A NARROW SELECTION OF GRAPHIC FILE TYPES. IF
#   THE INCLUDED SAMPLE FILES WORK BUT YOUR OWN GRAPHICS DON'T, YOU MAY NEED TO ADD A NEW TYPE OF
#   MODE HANDLING TO THE GetAdjustedPixel() FUNCTION.

import Image
import sys


# CHECK PIXEL'S COLOR AND RETURN AS A STRING IN THE FORMAT "turf_rval_gval_bval"
def GetAdjustedPixel(myImage, x, y):
	#STANDARDIZE COLOR STORAGE MODE TO R,G,B FORMAT
	if myImage.mode == 'P':
		red, green, blue = im.getpixel((x, y)), im.getpixel((x, y)), im.getpixel((x, y))
	elif myImage.mode == 'RGB':
		red, green, blue = im.getpixel((x, y))
	else:
		raise ValueError, "Unsupported palette mode: " + myImage.mode

	colorkey = "turf_" + str(red) + "_" + str(green) + "_" + str(blue)
	return colorkey


im = Image.open(sys.argv[1])
outMap = open(sys.argv[2] + '.dmp', 'w')
outStubs = open(sys.argv[2] + '_turf_stubs.dm', 'w')

print 'Mode information: ' + im.format + "; " + str(im.size) + "; " + im.mode

colors = {}
weights = {}
holdOutput = []
symbolValue = 97 # ASCII a
width, height = im.size

for y in range(height):
	print "Processing " + str(y)
	dataLine = ''

	for x in range(width):
		colorkey = GetAdjustedPixel(im, x, y)

		if not(colors.has_key(colorkey)):
			weights[colorkey] = 1
			colors[colorkey] = chr(symbolValue)
			symbolValue += 1
		else:
			weights[colorkey] = weights[colorkey] + 1

		dataLine += colors[colorkey]

	dataLine += "\n"
	# MAP OUTPUT GOES TO holdOutput LIST TEMPORARILY BECAUSE ALL TURF TYPES WON'T BE KNOWN
	# UNTIL END OF PROCESSING
	holdOutput.append(dataLine)

outStubs.write('turf\n')
for aKey in weights.keys():
	print aKey, weights[aKey], colors[aKey]
	outMap.write('"' + colors[aKey] + '" = (/turf/' + aKey + ')\n')
	outStubs.write('\t' + aKey + '\n')
	outStubs.write("\t\ticon = 'mapdump.dmi'\n")
	outStubs.write('\t\ticon_state = "' + colors[aKey] + '"\n')
outStubs.close()

outMap.write('\n(1,1,1) = {"\n')
for dataLine in holdOutput:
	outMap.write(dataLine)
outMap.write('"}')
outMap.close()
